#include <qfiledlg.h>
#include <qfile.h>
#include <kapp.h>
#include <ktoolbar.h>
#include <kmenubar.h>
#include <kiconloader.h>
#include <qmsgbox.h>
#include <qstring.h>
#include "edit.h"

// Define generic command codes
#define ID_OPEN 100
#define ID_NEW 101
#define ID_SAVE 102
#define ID_SAVEAS 103
#define ID_CLOSE 104
#define ID_NEWWINDOW 105
#define ID_CLOSEWINDOW 106
#define ID_COPY 107
#define ID_CUT 108
#define ID_PASTE 109
#define ID_OPTIONS 110
#define ID_EXIT 111
#define ID_HELPCONTENTS 112
#define ID_ABOUT 113

// Define status item codes
#define ID_HINTTEXT 300

Edit::Edit()
  : filename_()
{
  initMenuBar();
  initToolBar();
  initStatusBar();

  view= new QMultiLineEdit(this, "Main View");
  connect(view, SIGNAL(textChanged()), SLOT(textChanged()));
  setView(view);
  show();

  enableCommand(ID_OPEN);
  enableCommand(ID_EXIT);
  disableCommand(ID_CLOSE);
  disableCommand(ID_SAVE);
  disableCommand(ID_SAVEAS);

  modified= false;
}

Edit::~Edit()
{

}

void Edit::help()
{
  kapp->invokeHTMLHelp("", "");
}

void Edit::commandCallback(int id_)
{
  QString name;

  switch(id_) {
  case ID_NEW:
    view->clear();
    filename_= "";
    break;
  case ID_OPEN:
    name= QFileDialog::getOpenFileName();
    if (!name.isEmpty())
      load(name);
    break;
  case ID_SAVE:
    if (!filename_.isEmpty())
      saveAs(filename_);
    else {
      name= QFileDialog::getSaveFileName();
      if (!name.isEmpty())
        saveAs(name);
    }
    break;
  case ID_SAVEAS:
    name= QFileDialog::getSaveFileName();
    if (!name.isEmpty())
      saveAs(name);
    break;
  case ID_HELPCONTENTS:
    help();
    break;
  case ID_ABOUT:
    QMessageBox::about(this, "About Edit",
		       "This is a simple text editor example program.");
    break;
  case ID_EXIT:
    if (exit())
      ::exit(0);
    break;
  }
}

void Edit::textChanged()
{
  modified= true;
  enableCommand(ID_SAVE);
  enableCommand(ID_SAVEAS);
}

void Edit::initMenuBar()
{
  QPopupMenu *file = new QPopupMenu;

  file->insertItem(klocale->translate("Open..."), ID_OPEN );
  file->insertItem(klocale->translate("New..."), ID_NEW );
  file->insertItem(klocale->translate("Save"), ID_SAVE);
  file->insertItem(klocale->translate("Save As..."), ID_SAVEAS);
  file->insertItem(klocale->translate("Close"), ID_CLOSE);
  file->insertSeparator();
  file->insertItem(klocale->translate("Exit"), ID_EXIT);

  QPopupMenu *help = new QPopupMenu;
  help->insertItem( klocale->translate("Contents..."), ID_HELPCONTENTS);
  help->insertItem( klocale->translate("About..."), ID_ABOUT);

  menu = new KMenuBar( this );
  CHECK_PTR( menu );
  menu->insertItem( klocale->translate("&File"), file );
  menu->insertSeparator();
  menu->insertItem( klocale->translate("&Help"), help );
  menu->show();
  setMenu(menu);

  // Connect things together
  connect (file, SIGNAL (activated (int)), SLOT (commandCallback (int)));
  connect (help, SIGNAL (activated (int)), SLOT (commandCallback (int)));
}

void Edit::initToolBar()
{
  KIconLoader *loader = kapp->getIconLoader();
  toolbar = new KToolBar(this);

  toolbar->insertButton(loader->loadIcon("filenew.xpm"),
			ID_NEW, TRUE,
			klocale->translate("Create a new file"));
  toolbar->insertButton(loader->loadIcon("fileopen.xpm"),
			ID_OPEN, FALSE, 
			klocale->translate("Open a file"));
  toolbar->insertButton(loader->loadIcon("filefloppy.xpm"),
			ID_SAVE, FALSE, 
			klocale->translate("Save the current file"));

  addToolBar(toolbar);
  toolbar->setBarPos(KToolBar::Top);
  toolbar->show();
  connect(toolbar, SIGNAL(clicked(int)), SLOT(commandCallback(int)));
}

void Edit::initStatusBar()
{
  statusbar= new KStatusBar(this);
  statusbar->insertItem("Welcome to Edit", ID_HINTTEXT);
  statusbar->show();
  setStatusBar(statusbar);
}

void Edit::setHint(const char *text)
{
  statusbar->changeItem(text, ID_HINTTEXT);
}

void Edit::closeEvent(QCloseEvent *e)
{
  // Catch if modified
  if (exit())
    e->accept();
}

int Edit::exit()
{
  int die= 0;

  if (!modified)
    die= 1;
  else
    if (QMessageBox::warning(this, klocale->translate("Unsaved Changes"),
			      "You have unsaved changes, you will loose them "
			      "if you exit now.",
			      "Exit", "Cancel",
			      0, 1, 1))
      die= 0;
    else
      die= 1;

  return die;
}

void Edit::enableCommand(int id) //SLOT
{
  toolbar->setItemEnabled(id, true);
  menu->setItemEnabled(id, true);
}

void Edit::disableCommand(int id) // SLOT
{
  toolbar->setItemEnabled(id, false);
  menu->setItemEnabled(id, false);
}

void Edit::saveAs(const char *filename) {
    QFile f( filename );
    if ( !f.open( IO_WriteOnly ) )
	return;

    QTextStream t(&f);

    t << view->text();

    f.close();
    modified= false;
    filename_= filename;
}

void Edit::load(const char *filename) {
    QFile f( filename );
    if ( !f.open( IO_ReadOnly ) )
	return;

    view->setAutoUpdate( FALSE );
    view->clear();

    QTextStream t(&f);
    while ( !t.eof() ) {
	QString s = t.readLine();
	view->append( s );
    }
    f.close();

    view->setAutoUpdate( TRUE );
    view->repaint();
    modified= false;
    filename_= filename;
}

#include "edit.moc"
